package me.Destro168.FC_Announcer;

import me.Destro168.FC_General.ColorLib;
import me.Destro168.FC_General.LocationInsideAreaCheck;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;

public class AnnouncementManager
{
	private FileConfiguration config;
	private FC_Announcer plugin;
	
	private AnnouncementGroup ag;
	private int[] currentLine;
	
	private int[] tid;
	
	private boolean displayAnnouncementsInConsole;
	private String broadcastTag;
	
	public void setTaskId(int x, int y) { tid[x] = y; }
	
	public AnnouncementGroup getAnnouncementGroup() { return ag; }
	public int getTaskId(int x) { return tid[x]; }
	
	public AnnouncementManager()
	{
		//Store plugin
		plugin = FC_Announcer.plugin;
		
		//Get config.
		config = plugin.getConfig();
		
		//Create the new announcement group.
		ag = new AnnouncementGroup();
		
		//Get the display announcements in console variable.
		displayAnnouncementsInConsole = config.getBoolean("Setting.displayAnnouncementsInConsole");
		
		//Set broadcast tag.
		broadcastTag = config.getString("Setting.BroadcastTag");
	}
	
	public void reload()
	{	
		int ac = 0;
		
		ag.initialize();
		
		ac = ag.getAnnouncementCount();
		
		currentLine = new int[ac];
		
		tid = new int[ac];
		
		for (int i = 0; i < ac; i++)
		{
			currentLine[i] = 0;
			tid[i] = 0;
		}
		
		//Cancel all announcements.
		Bukkit.getScheduler().cancelTasks(plugin);
		
		//Start announcing announcements.
		beginAnnouncing();
	}
	
	//Starts announcements that are stored as "enabled" in the config.
	public void beginAnnouncing()
	{
		//Get configuration file
		config = plugin.getConfig();
		
		//Handle AutoAnnounce if the option is enabled.
		if (config.getBoolean("Setting.autoEnable") == true)
			ag.autoEnable();

		//If there is one announcement, then announce
		if (ag.getAnnouncementCount() > -1)
		{
			//For all announcements that are enabled, create the processes and begin them for all groups.
			for (int i = 0; i < ag.getAnnouncementCount(); i++)
			{
				if (ag.getIsActive(i) == true)
				{
					tid[i] = startAnnouncement(tid[i], i);
				}
			}
		}
		else
		{
			plugin.getLogger().info("There are no announcements to display.");
		}
	}
	
	//Starts a new announcement task.
	public int startAnnouncement(int taskID, int group)
	{
		//Get configuration file
		config = plugin.getConfig();
		
		//Create a final group to be broadcast.
		final int currentGroup = group;
		
		Bukkit.getScheduler().cancelTask(taskID);
		
		//Create the task and store the task id.
		taskID = Bukkit.getServer().getScheduler().scheduleSyncRepeatingTask(plugin, new Runnable()
		{	
			@Override
			public void run() 
			{
				broadcastMessage(currentGroup);
			}	
		}, 0, ag.getInterval(group) * 20);
		
		//returns the taskID
		return taskID;
	}
	
	public void broadcastMessage(int group) 
	{	
		//Get configuration file
		config = plugin.getConfig();
		
		//Variables
		String line;
		Location boundOne;
		Location boundTwo;
		LocationInsideAreaCheck liac;
		ColorLib colorLib = new ColorLib();
		AnnouncerPerms ap;
		
		//Set our line.
		line = ag.getLine(group, currentLine[group]);
		
		//If our line is blank, then...
		if (line == null || line.equals(""))
		{
			//Set the current group for the announcement group to 0.
			currentLine[group] = 0;
			
			//Update line
			line = ag.getLine(group, currentLine[group]);
		}
		
		if (displayAnnouncementsInConsole == true)
			plugin.getLogger().info("[" + String.valueOf(group) + "]: " + line);
		
		//If the line is equal to null still, that means no announcements, so discontinue execution.
		if (line == null)
		{
			plugin.getLogger().info("No announcements, please disable all groups and autoenable if you don't want to see this message. Or add announcements :)");
		}
		else
		{
			//Set colors.
			line = colorLib.parseColors(broadcastTag + line); //this.log.info("Line: " + line)
			
			//Message the announcement to all players.
			for (Player player: plugin.getServer().getOnlinePlayers()) 
		    {
				ap = new AnnouncerPerms();
				ap.setPlayer(player);
				
				//Check if they have the view permission.
				if (ap.viewGroup(group) == true)
				{
					//If the player has the ignore permission, then we want to not show them it.
					if (ap.ignoresGroup(group) == true)
					{
						//If the player isn't an op AND they have the ignore permission.
						if (ap.isAdmin() == false)
							return;
					}
					
					//Make sure that it is in proper zone.
					
					//Create two locations to store the zone bounds.
					boundOne = new Location(player.getWorld(), ag.getBlock1X(group), ag.getBlock1Y(group), ag.getBlock1Z(group));
					boundTwo = new Location(player.getWorld(), ag.getBlock2X(group), ag.getBlock2Y(group), ag.getBlock2Z(group));
					
					liac = new LocationInsideAreaCheck(player.getLocation(), boundOne, boundTwo);
					
					if (liac.getIsInside() == true)
						player.sendMessage(line);
				}
		    }
			
			currentLine[group]++;
		}
	}
	
	//Basically creates default settings for when the plugin first runs.
	public void handleConfiguration()
	{
		//Get configuration file
		FileConfiguration config = plugin.getConfig();
		
		//Update config files to 4.0
		if (config.getDouble("Version") < 4.0)
		{
			//Header for configuration file
			config.options().header("These are configuration variables");
			
			//Set the new version
			config.set("Version", 4.0);
			
			//Set default broadcaster tag.
			config.set("Setting.BroadcastTag", "&b[&3FC_Announcer&b]&f ");
			
			//Enable the feature automatic enable.
			config.set("Setting.autoEnable", true);
			
			//Set announcements to be displayed in the console.
			config.set("Setting.displayAnnouncementsInConsole", true);
			
			//Remove outdated settings.
			if (config.getString("BroadcastTag") != null)
			{
				//Transfer settings to new settings
				config.set("Setting.BroadcastTag", config.getString("BroadcastTag"));
				config.set("Setting.autoEnable", config.getBoolean("autoEnable"));
				config.set("Setting.displayAnnouncementsInConsole", config.getBoolean("displayAnnouncementsInConsole"));
				
				//Set old settings to null
				config.set("configCreated", null);
				config.set("version", null);
				config.set("BroadcastTag", null);
				config.set("autoEnable", null);
				config.set("displayAnnouncementsInConsole", null);
				
				//Remove total lines from all announcement groups.
				for (int i = 0; i < 1000; i++)
				{
					if (config.getBoolean("Announcement." + i + ".isCreated") == true)
						config.set("Announcement." + i + ".totalLines", null);
				}
			}
			
			//Create a new announcement if there is no announcement at 0,0
			if (config.getString("Announcement." + 0 + "." + 0) == null)
				ag.createNewAnnouncement(0, 0, "&bThis is the default UltraAnnouncer announcement! Type &3/announcer&b for help!");
		}
		
		//Upgrade to 4.1
		if (config.getDouble("Version") < 4.1)
		{
			//Set the new version
			config.set("Version", 4.1);
		}
		
		//Save config
		plugin.saveConfig();
	}
}
